%Calculates the Seifert matrix of a braid.  The input is the braid
%representation of the knot in one of two forms:
%1) A vector, where a number i means that the string i crosses over string i+1, 
%and -i means that i crosses under string i+1.
%For example, the braid [1 1 1] is the right handed trefoil and [1 -2 1 -2] is the figure eight.
%2) A character string where uppercase means a right-handed crossing 
%and lowercase means left-handed, e.g. 'AAA' is the right handed trefoil
%and 'AbAb' is the figure eight.

function Seifert3

x=input('Type in a braid representation: ');

%We check to see if the input is alphabetical, and if so we run the program
%'conversion' to translate it into a vector.
if ischar(x)==1
    x=conversion(x);
end

if length(find(x)) ~= length(x)
    error('Zero is not a valid data entry. Please try again.')
end

%We create a new vector x2 which has the braid elements ordered into their
%disjoint components.  To find what the components are we run the
%auxilliary program 'components'.
component=components(x);
k=0;
for i=1:size(component,1)
    for j=1:size(component,2)
        k=k+1;
        x2(k)=component(i,j);
    end
end

%We delete any zero entries that may have appeared in x2.
k=0;
for i=1:length(x2)
    if x2(i-k)==0
        k=k+1;
        x2(i-k+1)=[];
    end
end

x=x2; %We now work with this new ordered braid.

%Next we calculate how many components the link has.  We do this by running
%the auxilliary program 'link_number'.
count=link_number(x);


%We next find where the homology generators are.  For each crossing (i.e.
%each entry in the braid vector) we look through the braid vector until we
%find the next value with the same modulus, as this denotes the next
%crossing between the same two strands.  The final entries in the vector h
%will tell us that there are homology generators between crossings x(i) and
%x(h(i)).

h=[];

for i=1:length(x)-1
    h(i)=0;
    j=i;
    while (h(i)==0)&&(j<length(x))
        j=j+1;
        if abs(x(i))==abs(x(j))
            h(i)=j;
        end
    end 
end

lh=length(h);
M=[];

%We compute the Seifert matrix.  Refer to the documentation (sections 3.2
%and 3.3 for explanation).
for i=1:lh
    if h(i)~=0
        for j=i:lh
            if i==j
                M(i,j)=-sign(x(i) + x(h(i)));
            elseif (h(i)>h(j))
                M(i,j)=0;
                M(j,i)=0;
            elseif(h(i)<j)
                M(i,j)=0;
                M(j,i)=0;
            elseif (h(i)==j)
                if x(j)>0
                    M(i,j)=0;
                    M(j,i)=1;
                else
                    M(i,j)=-1;
                    M(j,i)=0;
                end
            elseif (abs(abs(x(i))-abs(x(j)))>1)
                M(i,j)=0;
                M(i,j)=0; 
            elseif (abs(x(i))-abs(x(j))==1)
                    M(i,j)=0;
                    M(j,i)=-1;
            elseif (abs(x(j))-abs(x(i))==1)
                    M(i,j)=1;
                    M(j,i)=0;
            else %debug output
                M(i,j)=3;
                M(j,i)=3;
            end
        end
    else
        M(:,i)=0;
        M(i,:)=0;
    end
end

%We now need to delete the rows and columns of zeros which appeared
%whenever h(i) was zero (i.e. there was no homology generator).
k=0;
for i=1:lh
    if h(i)==0
        k=k+1;
        M(:,i-k+1)=[];
        M(i-k+1,:)=[];
    end
end

disp('The Seifert matrix of this braid is') 
M

Msize=size(M,1);

%Now we calculate the genus of each disconnected component.  This we do
%using the formula g = 2 - #link components + braid length - #strands, and
%we need to call on the auxilliary program 'smallest_braid' to put the
%sub-braids into their smallest representation.
for a=1:size(component,1)
    if component(a,:)== zeros(1,size(component,2))
        g(a)=0;
    else
        g(a)= (2 - link_number(smallest_braid(component(a,:))) + length(find(component(a,:))) - (max(abs(smallest_braid(component(a,:)))+1)))/2;
    end
end

if count==1
    fprintf('This braid is a knot, ')
else
    fprintf('This braid is a link of %i components, ', count)
end

%The genus of the Seifert surface is the sum of the genera of the connected
%components.
genus=0;
for i=1:length(g)
    genus=genus+g(i);    
end

fprintf('it has genus %i ', genus)
fprintf('and its Seifert surface has %i disconnected component(s). \r', length(g))

%If you have the symbolic toolbox enabled in Matlab you can use the
%following two lines of code to find the Alexander polynomial instead 
%of using the polynomial fitting algorithm beneath. 
%t=sym('t');
%Alexander_polynomial=det(M-t*M')


if size(component,1)>1
    disp('The Alexander polynomial is zero.')
else
    for i=1:Msize+1
        alex(i)=det(M-(i-1)*M');
    end

tee=linspace(0, Msize, Msize+1);
Alex_coeffs=polyfit(tee,alex, Msize);
disp('The coefficients of the Alexander polynomial are')
Alex_int=round(Alex_coeffs)
end

%We calculate the signature of the link using the auxilliary program
%'signature'.  This computes the signature of the matrix M + M'.
sig=signature(M,-1);
fprintf('The signature is %i.  ', sig)

%We calculate the Arf invariant if the input is a knot.  This is done
%using the result that the Arf invariant is zero if and only if the
%Alexander polynomial evaluated at -1 is plus or minus one modulo 8.
if size(component,1)==1
    polyvalue=0;
    for i=1:length(Alex_int)
        polyvalue = polyvalue + Alex_int(i)*(-1)^(i-1);
    end
    if mod(polyvalue,8) == 1
        arf=0;
    elseif mod(polyvalue,8) == 7
        arf=0;
    else arf=1;
    end
    fprintf('The Arf invariant is %i. \r', arf)
end